from typing import Callable, Any, TypeAliasType, Iterator
from types import ModuleType
import inspect


def has_annotations(f: Callable, param_types: list[object], return_type: object = None) -> bool:
    a = f.__annotations__
    ret = a['return'] if 'return' in a else None
    params = list(a.values())[:-1] if 'return' in a else list(a.values())
    for param, param_type in zip(params, param_types):
        if isinstance(param, TypeAliasType):
            param = param.__value__
        if isinstance(param_type, TypeAliasType):
            param_type = param_type.__value__
        if param != param_type:
            return False
    return ret == return_type


def has_no_annotations(f: Callable) -> bool:
    return len(f.__annotations__) == 0


def imported_modules_of(module: ModuleType) -> Iterator[ModuleType]:
    return map(lambda m: m[1], filter(lambda m: inspect.ismodule(m[1]), inspect.getmembers(module)))


def eidp_test(message: str, minus_points: float) -> Callable[[Callable[..., Any]], Callable[..., Any]]:
    def wrapper(test: Callable[..., Any]) -> Callable[..., Any]:
        test.message = message
        test.minus_points = minus_points
        test.is_test = True
        return test
    return wrapper


def get_tests(module: ModuleType) -> dict[str, Callable[..., Any]]:
    return dict([(name, f) for name, f in module.__dict__
                 if callable(f) and hasattr(f, "is_test")])
